-- Puffin: A Tojaso FowlPlay Project
-- Model component for monitoring spell casting for player and target, plus mirror timers

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Brokers = {}
local clockTime = 0
local startTime = GetTime()
local lastTime = 0
local LDB, LQT
local menuBroker, clockBroker, equipmentBroker, xpBroker, mailBroker, goldBroker, systemBroker
local slots = { "Head", "Shoulder", "Chest", "Waist", "Legs", "Feet", "Wrist", "Hands", "MainHand", "SecondaryHand", }
local activeTooltip, scanTooltip
local startGold = 0

local function GetTimeText(zone, military, ampm)
	local t = ""
	local h, m = GetGameTime()
	if zone == "local" then local d = date("*t"); h = d.hour end
	if zone == "session" then
		local s = GetTime() - startTime; h = math.floor(s / 3600); m = math.floor((s - (h * 3600)) / 60)
		if h > 0 then t = string.format("%d:%02d:%02d", h, m, math.floor(s - (h * 3600) - (m * 60)))
		else t = string.format("%d:%02d", m, math.floor(s - (m * 60))) end
	else	
		if military then
			t = string.format ("%02d:%02d", h, m)
		else
			if ampm then ampm = " AM" end
			if h >= 12 then h = h - 12; if ampm then ampm = " PM" end end
			if h == 0 then h = 12 end
			t = string.format ("%d:%02d%s", h, m, ampm or "")
		end
	end
	return t
end

local function GetDurability()
	local durability, repairCost = 1, 0
	for _, slot in ipairs(slots) do
		local id = GetInventorySlotInfo(slot .. "Slot")
		local hasItem, _ ,cost = scanTooltip:SetInventoryItem("player", id)
		if hasItem then repairCost = repairCost + cost end
	end
	for slotId = 1, 18 do
		if GetInventoryItemLink("player", slotId) then
			local x, y = GetInventoryItemDurability(slotId)
			if x and y > 0 then if x / y < durability then durability = x / y end end
		end
	end
	return string.format("%d%%", durability * 100), repairCost
end

local function HideTooltip(tooltip)
	if not MouseIsOver(tooltip) then
		local frame = tooltip._activeFrame
		if frame and frame.tooltip then
			tooltip._activeFrame = nil
			tooltip:SetScript('OnLeave', nil)
			LQT:Release(tooltip)
			frame.tooltip = nil; activeTooltip = nil
		end
	end
end

local function AddLine(text, f, noCombat)
	if noCombat and InCombatLockdown() then
		activeTooltip:AddLine(string.format("|c80808000%s|r", text))
	else
		local line = activeTooltip:AddLine(text)
		activeTooltip:SetLineScript(line, "OnMouseUp", f)
	end
end

-- Initialize by registering for relevant events
function Model_Brokers.Configure()
	LDB = LibStub("LibDataBroker-1.1", true)
	assert(LDB, EF("Model_Brokers", "LibDataBroker not found"))
	LQT = LibStub("LibQTip-1.0")
	assert(LQT, EF("Model_Brokers", "LibQTip not found"))
	scanTooltip = CreateFrame("GameTooltip", "Puffin_ScanTip", nil, "GameTooltipTemplate")
	scanTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	startGold = GetMoney()
	
	menuBroker = LDB:NewDataObject("Puffin Menu", {
		type = "launcher", text = "Puffin", label = "Puffin", icon = [[Interface\AddOns\Puffin\Media\Puffin]],
		OnClick = function(_, msg) if msg == "LeftButton" then Puffin.OptionsPanel() end end,
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_MenuTooltip", 1, "LEFT")
				frame.tooltip = tooltip; activeTooltip = tooltip
				tooltip:AddHeader("|cffffcc00Puffin|r")
				AddLine(CHARACTER_BUTTON, function() securecall(ToggleCharacter, "PaperDollFrame") end)			
				AddLine(SPELLBOOK_ABILITIES_BUTTON, function()
						if not InCombatLockdown() then ToggleSpellBook(BOOKTYPE_SPELL) end
					end, true)			
				AddLine(TALENTS_BUTTON, function()
						if (not PlayerTalentFrame) then LoadAddOn('Blizzard_TalentUI') end
						PlayerTalentFrame_Toggle()
					end)		
				AddLine(ACHIEVEMENT_BUTTON, function() securecall(ToggleAchievementFrame) end)			
				AddLine(GUILD, function() ToggleGuildFrame() end)			
				AddLine(SOCIAL_BUTTON, function() securecall(ToggleFriendsFrame, 1) end)			
				AddLine(COLLECTIONS, function()
						if (not CollectionsJournal) then LoadAddOn('Blizzard_Collections') end
						securecall(ToggleCollectionsJournal)
					end, true)			
				AddLine(GROUP_FINDER, function() securecall(PVEFrame_ToggleFrame) end)			
				AddLine(ENCOUNTER_JOURNAL, function() securecall(ToggleEncounterJournal) end)			
				AddLine("Calendar", function() securecall(ToggleCalendar) end)			
				AddLine("Time Manager", function() securecall(ToggleTimeManager) end)			
				AddLine(GAMEMENU_HELP, function() securecall(ToggleHelpFrame) end)			
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame)
			if LQT then
				if frame.tooltip then
					frame.tooltip._activeFrame = frame
					if MouseIsOver(frame.tooltip) then
						frame.tooltip:SetScript("OnLeave", HideTooltip)
					else
						HideTooltip(frame.tooltip)
					end
				end
			end
		end
	})

	clockBroker = LDB:NewDataObject("Puffin Clock", {
		type = "data source", text = "Clock", label = "Clock", icon = GetItemIcon(4389),
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_ClockTooltip", 1, "LEFT")
				frame.tooltip = tooltip
				local _, _, clockIcon = GetSpellInfo(129772)
				tooltip:AddHeader("|cffffcc00Clock|r")
				tooltip:AddLine(string.format("%s", date("%m-%d-%Y")))
				tooltip:AddLine(string.format("%s |cffffff00(Local Time)|r", GetTimeText("local", false, true)))
				tooltip:AddLine(string.format("%s |cffffff00(Server Time)|r", GetTimeText("server", false, true)))
				tooltip:AddLine(string.format("|T%s:16|t |cffffff00%s Session|r", clockIcon, GetTimeText("session", false, false)))
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame) if LQT then LQT:Release(frame.tooltip) end end
	})

	equipmentBroker = LDB:NewDataObject("Puffin Equipment", {
		type = "data source", text = "Equipment", label = "Equipment", icon = GetItemIcon(31823),
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_EquipmentTooltip", 1, "LEFT")
				frame.tooltip = tooltip
				local durability, repairCost = GetDurability()
				tooltip:AddHeader("|cffffcc00Equipment|r")
				tooltip:AddLine("|cffffff00Durability|r " .. durability)
				tooltip:AddLine(string.format("|cffffff00Repair Cost|r %s ", GetCoinTextureString(repairCost)))
				local total, equipped = GetAverageItemLevel()
				if total and equipped then tooltip:AddLine(string.format("|cffffff00Item Level|r %d/%d", total, equipped)) end
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame) if LQT then LQT:Release(frame.tooltip) end end
	})

	xpBroker = LDB:NewDataObject("Puffin Experience", {
		type = "data source", text = "XP", label = "XP", icon = GetItemIcon(29750),
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_XPTooltip", 1, "LEFT")
				frame.tooltip = tooltip
				tooltip:AddHeader("|cffffcc00Experience|r")
				local curXP = UnitXP("player")
				local maxXP = UnitXPMax("player")
				local restXP = GetXPExhaustion() or 0
				tooltip:AddLine(string.format("|cffffff00Percent XP|r %d%%", (curXP / maxXP) * 100))
				tooltip:AddLine(string.format("|cffffff00Current XP|r %d", curXP))
				tooltip:AddLine(string.format("|cffffff00Maximum XP|r %d", maxXP))
				tooltip:AddLine(string.format("|cffffff00Rested|r %d%%", (restXP / maxXP) * 100))
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame) if LQT then LQT:Release(frame.tooltip) end end
	})
	
	mailBroker = LDB:NewDataObject("Puffin Mail", {
		type = "data source", text = "Mail", label = "Mail", icon = GetItemIcon(94553),
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_MailTooltip", 1, "LEFT")
				frame.tooltip = tooltip
				tooltip:AddHeader("|cffffcc00Mail|r")
				if HasNewMail() then tooltip:AddLine("|cffffff00You have mail!|r") else tooltip:AddLine("|cffffff00No new mail|r") end
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame) if LQT then LQT:Release(frame.tooltip) end end
	})

	goldBroker = LDB:NewDataObject("Puffin Gold", {
		type = "data source", text = "Gold", label = "Gold", icon = GetItemIcon(43639),
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_GoldTooltip", 1, "LEFT")
				frame.tooltip = tooltip
				tooltip:AddHeader("|cffffcc00Gold|r")
				local currentGold = GetMoney()
				tooltip:AddLine(GetCoinTextureString(currentGold))
				local change = currentGold - startGold
				if change >= 0 then
					tooltip:AddLine("|cffffff00Session Profit|r " .. GetCoinTextureString(change))
				else
					tooltip:AddLine("|cffffff00Session Loss|r " .. GetCoinTextureString(-change))
				end
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame) if LQT then LQT:Release(frame.tooltip) end end
	})
	
	systemBroker = LDB:NewDataObject("Puffin System", {
		type = "data source", text = "0", label = "FPS", icon = GetItemIcon(89990),
		OnEnter = function(frame)
			if LQT then
				local tooltip = LQT:Acquire("Puffin_SystemTooltip", 1, "LEFT")
				frame.tooltip = tooltip
				local _, _, latencyHome, latencyWorld = GetNetStats()
				local version, internalVersion = GetBuildInfo()
				local _, _, wowIcon = GetSpellInfo(100951)
				local mem, addons = 0, 0
				for i = 1, GetNumAddOns() do
					local _, _, _, enabled = GetAddOnInfo(i)
					if enabled then addons = addons + 1; mem = mem + GetAddOnMemoryUsage(i) end
				end
				local resolution, api = GetCVar("gxFullscreenResolution"), GetCVar("gxApi")
				if api == "D3D11" then api = "DX11" elseif api == "D3D9" then api = "DX9" end
				tooltip:AddHeader("|cffffcc00Puffin System|r")
				local fps = math.floor(GetFramerate())
				tooltip:AddLine(string.format("|cffffff00FPS|r %d", fps))
				tooltip:AddLine(string.format("|cffffff00Latency|r %d ms (Home)", math.floor(latencyHome or 0)))
				tooltip:AddLine(string.format("|cffffff00Latency|r %d ms (World)", math.floor(latencyWorld or 0)))
				tooltip:AddLine(string.format("%d Addons Enabled (%0.2f MB)", addons, mem / 1000))
				tooltip:AddLine(string.format("%s Graphics (%s)", api, resolution))
				tooltip:AddLine(string.format("|T%s:16|t |cffffff00%s Build %d|r", wowIcon, version, internalVersion))
				tooltip:SmartAnchorTo(frame)
				tooltip:Show()
			end
		end,
		OnLeave = function(frame) if LQT then LQT:Release(frame.tooltip) end end
	})
end

-- Mirror bar events are not reliabe so check during updates
function Model_Brokers.OnUpdate()
	local elapsed = 0
	local now = GetTime()
	if lastTime > 0 then
		elapsed = now - lastTime
		clockTime = clockTime + elapsed
		if clockTime >= 2 then -- update data brokers about every other second
			clockTime = 0
			if clockBroker then clockBroker.text = GetTimeText("local", false, true) end
			if equipmentBroker then equipmentBroker.text = GetDurability() end
			if xpBroker then xpBroker.text = string.format("%d%%", (UnitXP("player") / UnitXPMax("player")) * 100) end
			if mailBroker then mailBroker.text = HasNewMail() and "You have mail!" or "No new mail" end
			if goldBroker then goldBroker.text = GetCoinTextureString(GetMoney()) end
			if systemBroker then systemBroker.text = math.floor(GetFramerate()) end
			if activeTooltip then HideTooltip(activeTooltip) end	
		end
	end
	lastTime = now
end

Puffin.RegisterCallback("Puffin_Configure", Model_Brokers.Configure)
Puffin.RegisterCallback("Puffin_Update", Model_Brokers.OnUpdate)
