-- Puffin: A Tojaso FowlPlay Project
-- Model component for chat tab, inspired by various chat addons (Prat, Chatter, PhanxChat)

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database

local Model_Chat = {}

local hideEvent = function(frame) frame:Hide() end -- handler for events that might show a hidden frame
local ignoreEvent = function() end -- handler for events that want to ignore

-- Hook chat frame show to keep bottom button hidden
local function Chat_OnShow(self)
	local b = _G[self:GetName() .. "ButtonFrameBottomButton"]
	if self:AtBottom() then b:Hide() else b:Show() end
end

-- Hook bottom button to keep it hidden
local function Bottom_OnClick(self)
	local f = self:GetParent()
	if f.ScrollToBottom then f:ScrollToBottom() else f:GetParent():ScrollToBottom() end
	f = _G[f:GetName() .. "ButtonFrameBottomButton"]; f:Hide()
end

-- Initialize chat frames from info in the configuration database
function Model_Chat.Configure()
	-- Global variables that control chat frame alpha, mouseover, and fading
	DEFAULT_CHATFRAME_ALPHA = 0.3
	CHAT_FRAME_TAB_SELECTED_MOUSEOVER_ALPHA = 1
	CHAT_FRAME_TAB_SELECTED_NOMOUSE_ALPHA = 0
	CHAT_FRAME_TAB_ALERTING_MOUSEOVER_ALPHA = 1
	CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = 0.5
	CHAT_FRAME_TAB_NORMAL_MOUSEOVER_ALPHA = 0.5
	CHAT_FRAME_TAB_NORMAL_NOMOUSE_ALPHA = 0
	CHAT_FRAME_FADE_OUT_TIME = 1
	CHAT_TAB_HIDE_DELAY = 0
	
	SetCVar("showTimestamps", "[%H:%M] ")
	
	-- Hide buttons common to all chat frames
	ChatFrameMenuButton:SetScript("OnShow", ChatFrameMenuButton.Hide); ChatFrameMenuButton:Hide()
	QuickJoinToastButton:SetScript("OnShow", QuickJoinToastButton.Hide); QuickJoinToastButton:Hide()

	-- Set up each chat frame's appearance
	for i = 1, NUM_CHAT_WINDOWS do
		local frame = _G["ChatFrame" .. i]
		-- hide buttons attached to the chat frame
		local name = frame:GetName()
		local f = _G[name .. "ButtonFrame"]; f.Show = ignoreEvent; f:Hide()
		f = _G[name .. "ButtonFrameUpButton"]; f.Show = ignoreEvent; f:Hide()
		f = _G[name .. "ButtonFrameDownButton"]; f.Show = ignoreEvent; f:Hide()
		
		f = _G[name .. "ButtonFrameBottomButton"] -- bottom button gets special treatment
		f:ClearAllPoints(); f:SetParent(frame); f:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, -4)
		f:SetAlpha(0.5); f:Hide()
		f:SetScript("OnClick", Bottom_OnClick) -- replace the bottom button's OnClick script handler
		frame:HookScript("OnShow", Chat_OnShow) -- make sure the bottom button gets hidden when chat frame is shown

		f = _G[name .. "EditBox"]; f:ClearAllPoints() -- adjust location of the edit box
		f:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", -13, -16)
		f:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", 0, -16)
		
		f = _G[name .. "EditBoxLeft"]; f.Show = ignoreEvent; f:Hide()
		f = _G[name .. "EditBoxRight"]; f.Show = ignoreEvent; f:Hide()
		f = _G[name .. "EditBoxMid"]; f.Show = ignoreEvent; f:Hide()
		f = _G[name .. "EditBoxFocusLeft"]:SetTexture(nil)
		f = _G[name .. "EditBoxFocusRight"]:SetTexture(nil)
		f = _G[name .. "EditBoxFocusMid"]:SetTexture(nil)
		
		local w = DB.GetAttribute("defaults", "chat", "width")
		local h = DB.GetAttribute("defaults", "chat", "height")
		SetChatWindowSavedDimensions(i, w, h)
		SetChatWindowSavedPosition(i, "BOTTOMLEFT", 0, 0)
		
		local font = DB.GetAttribute("defaults", "chat", "font")
		local fsize = DB.GetAttribute("defaults", "chat", "fontSize")
		local fflags = DB.GetAttribute("defaults", "chat", "fontFlags")
		frame:SetFont(font, fsize, fflags)
	end
end

Puffin.RegisterCallback("Puffin_Configure", Model_Chat.Configure)
